"use strict";
var _a, _b;
Object.defineProperty(exports, "__esModule", { value: true });
exports.BASE_URL = exports.req = void 0;
const axios_1 = require("axios");
const uris_1 = require("./uris");
const config_1 = require("./config");
const credentials_singleton_1 = require("./credentials-singleton");
// eslint-disable-next-line unicorn/numeric-separators-style
const MB = 1000000;
const USER_AGENT = 'Anypoint-CLI/' + require('../package.json').version;
const BASE_URL = `https://${((_b = (_a = credentials_singleton_1.CredentialsSingleton.getInstance()) === null || _a === void 0 ? void 0 : _a.getFlags()) === null || _b === void 0 ? void 0 : _b.host) || uris_1.DEFAULT_HOST}`;
exports.BASE_URL = BASE_URL;
const isValidUrl = (proxyKey) => {
    try {
        // eslint-disable-next-line no-new
        new URL(proxyKey);
        return true;
    }
    catch {
        return false;
    }
};
const axCnfg = {
    baseURL: `${BASE_URL}`,
    maxContentLength: MB * 200,
    maxBodyLength: MB * 200,
};
if (isValidUrl(process.env.HTTP_PROXY || '') || isValidUrl(process.env.HTTPS_PROXY || '')) {
    const urlProx = new URL(process.env.HTTPS_PROXY || process.env.HTTP_PROXY || '');
    axCnfg.proxy = {
        protocol: urlProx.protocol,
        host: urlProx.hostname,
        port: Number(urlProx.port),
        auth: {
            username: urlProx.username,
            password: urlProx.password,
        },
    };
}
const req = axios_1.default.create(axCnfg);
exports.req = req;
req.interceptors.request.use(async (cnfg) => {
    var _a, _b, _c, _d, _e, _f, _g;
    if ((_a = cnfg.baseURL) === null || _a === void 0 ? void 0 : _a.includes('anypoint.mulesoft')) {
        const session = config_1.default.get('session');
        const token = config_1.default.get('token');
        const env = `${cnfg.headers['X-ANYPNT-ENV-ID'] ? cnfg.headers['X-ANYPNT-ENV-ID'] : (((_b = session === null || session === void 0 ? void 0 : session.selectedEnvironment) === null || _b === void 0 ? void 0 : _b.id) || '')}`;
        const org = `${cnfg.headers['X-ANYPNT-ORG-ID'] ? cnfg.headers['X-ANYPNT-ORG-ID'] : (((_c = session === null || session === void 0 ? void 0 : session.selectedOrganization) === null || _c === void 0 ? void 0 : _c.id) || '')}`;
        const owner = ((_e = (_d = session === null || session === void 0 ? void 0 : session.account) === null || _d === void 0 ? void 0 : _d.user) === null || _e === void 0 ? void 0 : _e.id) || '';
        cnfg.baseURL = `https://${((_g = (_f = credentials_singleton_1.CredentialsSingleton.getInstance()) === null || _f === void 0 ? void 0 : _f.getFlags()) === null || _g === void 0 ? void 0 : _g.host) || uris_1.DEFAULT_HOST}`;
        cnfg.headers['X-ANYPNT-ENV-ID'] = env;
        cnfg.headers['X-ANYPNT-ORG-ID'] = org;
        cnfg.headers['x-organization-id'] = org;
        cnfg.headers['x-owner-id'] = owner;
        cnfg.headers.Authorization = (token === null || token === void 0 ? void 0 : token.token) ? `Bearer ${token.token}` : '';
        cnfg.headers['User-Agent'] = USER_AGENT;
    }
    return cnfg;
}, error => Promise.reject(error));
req.interceptors.response.use(async (response) => {
    return response;
}, error => {
    var _a;
    if (error.errno === -3008) {
        const msg = `The Anypoint Base URL provided "${error.hostname}" is not valid`;
        // Expected host parameter to be one of: ${validServers?.join(', ')}`
        error = new Error(msg);
    }
    else if (((_a = error.response) === null || _a === void 0 ? void 0 : _a.status) === 401) {
        const msg = `Authentication failed \n
    ›   Login to ${error.response.config.baseURL}${error.response.config.url} failed with \n
    ›   status 401 - Unauthorized`;
        error = new Error(msg);
    }
    return Promise.reject(error);
});
