"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const __1 = require("..");
const apollo = require('apollo-client');
const alink = require('apollo-link-http');
const acache = require('apollo-cache-inmemory');
const cross_fetch_1 = require("cross-fetch");
const login = require("../login");
const credentials_singleton_1 = require("../credentials-singleton");
const api_client_1 = require("../api-client");
const core_1 = require("./core");
class AuthenticationBaseCommand extends core_1.default {
    get req() {
        if (this._req)
            return this._req;
        this._req = api_client_1.req;
        return this._req;
    }
    get getGqlClient() {
        var _a;
        if (this._gqlClient === undefined) {
            try {
                const link = alink.createHttpLink({
                    uri: __1.uris.graphql(`https://${(_a = this.parsedFlags) === null || _a === void 0 ? void 0 : _a.host}`),
                    fetch: cross_fetch_1.default,
                });
                this._gqlClient = new apollo.ApolloClient({
                    cache: new acache.InMemoryCache(),
                    link: link,
                });
            }
            catch (error) {
                this.log('getGqlClient', error);
            }
        }
        return this._gqlClient;
    }
    // Query GraphQL
    // gqlQuery: GraphQL query wrapped in `graphql-tag`
    queryGql(gqlQuery, variables) {
        try {
            const client = this.getGqlClient;
            variables = variables || {};
            const token = __1.default.get('token');
            variables.accessToken = token.token;
            return client.query({
                query: gqlQuery,
                variables: variables,
            });
        }
        catch (error) {
            this.log('queryGql', error);
        }
    }
    async init() {
        try {
            const { flags } = await this.parse(this.constructor);
            credentials_singleton_1.CredentialsSingleton.getInstance().setFlags({ ...flags });
            await login.authenticate(flags);
            this.parsedFlags = flags;
        }
        catch (error) {
            this.error(error);
        }
    }
}
exports.default = AuthenticationBaseCommand;
