"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CoreFlags = exports.CollectMetricsFlag = exports.OutputFlag = exports.ProfileFlag = exports.HostFlag = exports.EnvironmentFlag = exports.OrganizationFlag = exports.BearerFlag = exports.ClientSecretFlag = exports.ClientIdFlag = exports.PasswordFlag = exports.UsernameFlag = exports.flagDescription = exports.BaseCommandFlagsEnum = void 0;
const config_1 = require("../config");
const uris_1 = require("../uris");
const core_1 = require("@oclif/core");
var BaseCommandFlagsEnum;
(function (BaseCommandFlagsEnum) {
    BaseCommandFlagsEnum["USERNAME"] = "username";
    BaseCommandFlagsEnum["PASSWORD"] = "password";
    BaseCommandFlagsEnum["CLIENT_ID"] = "client_id";
    BaseCommandFlagsEnum["CLIENT_SECRET"] = "client_secret";
    BaseCommandFlagsEnum["BEARER"] = "bearer";
    BaseCommandFlagsEnum["ORGANIZATION"] = "organization";
    BaseCommandFlagsEnum["ENVIRONMENT"] = "environment";
    BaseCommandFlagsEnum["HOST"] = "host";
    BaseCommandFlagsEnum["PROFILE"] = "profile";
    BaseCommandFlagsEnum["OUTPUT"] = "output";
    BaseCommandFlagsEnum["COLLECT_METRICS"] = "collectMetrics";
})(BaseCommandFlagsEnum = exports.BaseCommandFlagsEnum || (exports.BaseCommandFlagsEnum = {}));
const flagDescription = (envVariable, description) => `${description ? description + '. ' : ''}You can define this in the ${envVariable} environment variable.`;
exports.flagDescription = flagDescription;
exports.UsernameFlag = {
    [BaseCommandFlagsEnum.USERNAME]: core_1.Flags.string({
        description: (0, exports.flagDescription)('ANYPOINT_USERNAME', 'Username'),
        exclusive: ['client_id', 'token'],
        default: process.env.ANYPOINT_USERNAME || config_1.default.get('username'),
    }),
};
exports.PasswordFlag = {
    [BaseCommandFlagsEnum.PASSWORD]: core_1.Flags.string({
        description: (0, exports.flagDescription)('ANYPOINT_PASSWORD', 'Password'),
        dependsOn: ['username'],
        default: process.env.ANYPOINT_PASSWORD || config_1.default.get('password'),
    }),
};
exports.ClientIdFlag = {
    [BaseCommandFlagsEnum.CLIENT_ID]: core_1.Flags.string({
        description: (0, exports.flagDescription)('ANYPOINT_CLIENT_ID', 'Client ID'),
        exclusive: ['username', 'token'],
        default: process.env.ANYPOINT_CLIENT_ID || config_1.default.get('client_id'),
    }),
};
exports.ClientSecretFlag = {
    [BaseCommandFlagsEnum.CLIENT_SECRET]: core_1.Flags.string({
        description: (0, exports.flagDescription)('ANYPOINT_CLIENT_SECRET', 'Client Secret'),
        dependsOn: ['client_id'],
        default: process.env.ANYPOINT_CLIENT_SECRET || config_1.default.get('client_secret'),
    }),
};
exports.BearerFlag = {
    [BaseCommandFlagsEnum.BEARER]: core_1.Flags.string({
        description: (0, exports.flagDescription)('ANYPOINT_BEARER', 'Token Bearer'),
        exclusive: ['username', 'client_id'],
        default: process.env.ANYPOINT_BEARER,
    }),
};
exports.OrganizationFlag = {
    [BaseCommandFlagsEnum.ORGANIZATION]: core_1.Flags.string({
        description: (0, exports.flagDescription)('ANYPOINT_ORG', 'Organization Name'),
        default: process.env.ANYPOINT_ORG || config_1.default.get('organization'),
    }),
};
exports.EnvironmentFlag = {
    [BaseCommandFlagsEnum.ENVIRONMENT]: core_1.Flags.string({
        description: (0, exports.flagDescription)('ANYPOINT_ENV', 'Environment Name'),
        default: process.env.ANYPOINT_ENV || config_1.default.get('environment'),
    }),
};
exports.HostFlag = {
    [BaseCommandFlagsEnum.HOST]: core_1.Flags.string({
        description: (0, exports.flagDescription)('ANYPOINT_HOST', 'Host URL'),
        default: process.env.ANYPOINT_HOST || config_1.default.get('host') || uris_1.DEFAULT_HOST,
        // options: validServers,
        helpValue: 'anypoint.mulesoft.com',
    }),
};
exports.ProfileFlag = {
    [BaseCommandFlagsEnum.PROFILE]: core_1.Flags.string({
        description: 'Select Profile from credentials',
        default: 'default',
    }),
};
exports.OutputFlag = {
    [BaseCommandFlagsEnum.OUTPUT]: core_1.Flags.string({
        description: 'Format for commands output. Supported values are table (default) and json',
        default: 'table',
        char: 'o',
    }),
};
exports.CollectMetricsFlag = {
    [BaseCommandFlagsEnum.COLLECT_METRICS]: core_1.Flags.boolean({
        description: (0, exports.flagDescription)('COLLECT_METRICS', 'collect metrics'),
        default: process.env.COLLECT_METRICS || config_1.default.get('collectMetrics') || true,
    }),
};
exports.CoreFlags = {
    ...exports.UsernameFlag,
    ...exports.PasswordFlag,
    ...exports.ClientIdFlag,
    ...exports.ClientSecretFlag,
    ...exports.BearerFlag,
    ...exports.OrganizationFlag,
    ...exports.EnvironmentFlag,
    ...exports.HostFlag,
    ...exports.CollectMetricsFlag,
};
