"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getEnvOrg = exports.authenticate = exports.validateHttpProxies = void 0;
/* eslint-disable no-await-in-loop */
const _1 = require(".");
const config_1 = require("./config");
const fmt = require('util').format;
const _ = require('lodash');
const puppeteer = require('puppeteer');
const loginWithUsernameAndPassword = async (username, password) => {
    const body = {
        username: username,
        password: password,
    };
    const { data: res } = await _1.req.post(_1.uris.accountLogin, body);
    if (res.url && res.body) {
        try {
            // token.token = config.get('token.token')
            await getMe();
        }
        catch {
            const access_token = await loginWithMFA();
            return access_token;
        }
        // throw new Error('This user is configured with multi-factor authentication and requires additional factors beyond a password to log in.' +
        //       ' Please ask your Organization Administrator to mark it as exempt for use with the Anypoint CLI')
    }
    return res.access_token;
};
const loginWithClientIdAndSecret = async (client_id, client_secret) => {
    const body = {
        client_id: client_id,
        client_secret: client_secret,
        grant_type: 'client_credentials',
    };
    const { data: res } = await _1.req.post(_1.uris.accountToken, body);
    return res.access_token;
};
async function getMe() {
    const { data, } = await _1.req.get(_1.uris.userMe);
    return data;
}
const isAuthMechanismProvided = (credentials) => {
    let isProvided = false;
    if (credentials.username && credentials.password) {
        isProvided = true;
    }
    else if (credentials.client_id && credentials.client_secret) {
        isProvided = true;
    }
    else if (credentials.bearer) {
        isProvided = true;
    }
    return isProvided;
};
// eslint-disable-next-line no-promise-executor-return
const delay = (ms) => new Promise(resolve => setTimeout(resolve, ms));
const parseJwt = (token) => {
    return JSON.parse(Buffer.from(token.split('.')[1], 'base64').toString());
};
const loginWithMFA = async () => {
    const browser = await puppeteer.launch({ headless: false,
        defaultViewport: { width: 800, height: 600 },
        executablePath: process.env.CHROMIUM_PATH,
        args: ['--window-size=800,600', '--no-sandbox'] });
    const [page] = await browser.pages();
    await page.setViewport({ width: 800, height: 600 });
    await page.goto(`${_1.BASE_URL}/login/`);
    while (!page.url().includes('home/organizations')) {
        await delay(1000);
    }
    await page.goto(`${_1.BASE_URL}/apimanager/mulesoft-81901/#/organizations/`);
    await delay(3000);
    let cookies = await page.cookies();
    while (!cookies.some((c) => c.name === 'token')) {
        await delay(1000);
        cookies = await page.cookies();
    }
    const tknCookie = cookies.find((c) => c.name === 'token');
    const access_token = parseJwt(tknCookie.value).token;
    await browser.close();
    return access_token;
};
async function login(credentials) {
    const token = {
        tokenExpires: 60000,
        token: '',
        time: Date.now(),
    };
    const { bearer, username, password, client_id, client_secret } = credentials;
    if (isAuthMechanismProvided(credentials)) {
        if (bearer) {
            token.token = bearer;
        }
        else if (username && password) {
            const access_token = await loginWithUsernameAndPassword(username, password);
            token.token = access_token;
        }
        else if (client_id && client_secret) {
            const access_token = await loginWithClientIdAndSecret(client_id, client_secret);
            token.token = access_token;
        }
    }
    else {
        // This should never happen outside of tests - all the validation happens ahead of time in app_core.js
        throw new Error('No authentication mechanism was provided. Did you specify an empty string?');
    }
    // Set the Authorization header used in requests
    config_1.default.set('token', token);
    _1.req.defaults.headers.Authorization = `Bearer ${token.token}`;
    return true;
}
const loadOrganization = (organizations, identifier) => {
    const selectedOrganization = identifier ? _.find(organizations, { id: identifier }) ||
        _.find(organizations, { name: identifier }) : _.find(organizations, { isRoot: true });
    if (!selectedOrganization) {
        const msg = identifier +
            ' not found. You are a member of these orgs: ' +
            _.map(organizations, (org) => org.name + ' (' + org.id + ')').join(', ');
        throw new Error(msg);
    }
    return selectedOrganization;
};
const loadEnvironment = async (name) => {
    const envPromise = name ?
        _1.utils.getEnvironmentByName(name) :
        _1.utils.getDefaultEnvironment();
    const env = await envPromise;
    if (!env)
        return Promise.reject(new Error(fmt('\'%s\' environment not found in %s organization', name, config_1.default.get('session.selectedOrganization.name'))));
    return env;
};
/**
 * Logs warnings if the HTTP/HTTPS_PROXY env variables are set and appear invalid.
 * See https://www.mulesoft.org/jira/browse/SE-8952
 */
const validateHttpProxies = () => {
    for (const url of ['HTTPS_PROXY', 'HTTP_PROXY']) {
        const warning = validateEnvUrlSetting(url);
        if (warning) {
            console.warn(warning);
        }
    }
};
exports.validateHttpProxies = validateHttpProxies;
/**
 * Does a very basic url validation pass on process.env[key]
 */
const validateEnvUrlSetting = (proxyKey) => {
    const proxyUrl = process.env[proxyKey];
    const proxyUrlSet = proxyUrl && proxyUrl.length > 0;
    if (proxyUrlSet) {
        try {
            const { protocol, host } = new URL(proxyUrl);
            const proxyUrlLikelyInvalid = protocol || host;
            if (!proxyUrlLikelyInvalid) {
                return `Warning: The ${proxyKey} environment value: '${proxyUrl}' does not appear to be valid.`;
            }
        }
        catch {
            return `Warning: The ${proxyKey} environment value: '${proxyUrl}' does not appear to be valid.`;
        }
    }
    return null;
};
const authenticate = async (flags) => {
    var _a;
    (0, exports.validateHttpProxies)();
    if (await login(flags)) {
        const data = await getMe();
        const user = {
            id: '',
            username: '',
            firstName: '',
            lastName: '',
            email: '',
            createdAt: new Date(),
            contributorOfOrganizations: [],
            organization: { id: '' },
        };
        const account = {
            user: user,
            organizationPreferences: {
                orgId: '',
            },
        };
        const session = {
            account: account,
            selectedEnvironment: {
                id: '',
            },
            selectedOrganization: {
                id: '',
            },
        };
        session.account.user.id = data.id;
        session.account.organizationPreferences = data.organizationPreferences;
        session.account.user.organization.id = (_a = _.find(data.memberOfOrganizations, { isRoot: true })) === null || _a === void 0 ? void 0 : _a.id;
        config_1.default.set('session.account', account);
        const organization = loadOrganization(data.memberOfOrganizations, flags.organization);
        session.selectedOrganization = organization;
        config_1.default.set('session.selectedOrganization', organization);
        const env = await loadEnvironment(flags.environment);
        session.selectedEnvironment = env;
        config_1.default.set('session.selectedEnvironment', env);
    }
    else {
        throw new Error('Login fail');
    }
};
exports.authenticate = authenticate;
const getEnvOrg = async (org, env) => {
    const data = await getMe();
    const resp = {};
    const organization = loadOrganization(data.memberOfOrganizations, org);
    const enviroment = await _1.utils.getEnvironmentByName(env, org);
    resp.env = enviroment;
    resp.org = organization;
    return resp;
};
exports.getEnvOrg = getEnvOrg;
